@@ -18,10 +18,10 @@ http://fedc.wiwi.hu-berlin.de/xplore/ebooks/html/anr/anrhtmlframe62.html
 # pylint: disable-msg=W0142
 # pylint: disable-msg=E1101
 # pylint: disable-msg=E0611
-from statsmodels.compat.python import lzip, lfilter, callable, zip
+from statsmodels.compat.python import lzip, lfilter, zip
 import numpy as np
 import scipy.integrate
-from scipy.misc import factorial
+from statsmodels.compat.scipy import factorial
 from numpy import exp, multiply, square, divide, subtract, inf
 
 
